import pygame
import data
import math
import random
import pygame.sprite
import sound
from vector import Vector

GRAVITY = Vector(0.0, 0.00001)
ARROW_VELOCITY = Vector(-0.5, 0.0)

class Triggerable:
    def __init__(self, trigger):
        self.trigger = trigger
        self.triggered = False
        
    def update(self, dt, world):
        if not self.triggered:
            self.triggered = self.trigger.update(dt, world)
            
class Collidable:
    def collision_check(self, world):
        if world.player.alive and self.rect.colliderect(world.player.colliderect):
            return True
        return False
    
    def check_onscreen(self, world):
        if(self.position.x < -100 or self.position.x > world.width + 100):
            self.alive = False
        if(self.position.y > world.height + 100):
            self.alive = False
    

class Anvil(pygame.sprite.DirtySprite, Triggerable, Collidable):
    def __init__(self, startpos, trigger, velocity=Vector(0.0,0.0)):
        pygame.sprite.DirtySprite.__init__(self)
        Triggerable.__init__(self, trigger)
        self.position = startpos
        self.dirty = 2
        self.image, self.rect = data.load_image("enemies_anvil.PNG", [0, 255, 255])
        self.rect.center = (self.position.x, self.position.y)
        self.velocity = velocity.copy()
        self.alive = True
        
        
    def update(self, dt, world):
        if self.alive:
            Triggerable.update(self, dt, world)
            if self.triggered:
                self.velocity += GRAVITY * dt * 10
                self.position += self.velocity * dt
            
            self.rect.center = (self.position.x, self.position.y)
        
            if self.collision_check(world):
                world.player.change_state(world, world.player.states["dead"])
                
        self.check_onscreen(world)        
        
class Arrow(pygame.sprite.DirtySprite, Triggerable, Collidable):
    def __init__(self, startpos, trigger, velocity=Vector(-0.5, 0.0)):
        pygame.sprite.DirtySprite.__init__(self)
        Triggerable.__init__(self, trigger)
        self.dirty = 2
        self.position = startpos
        self.image, self.rect = data.load_image("enemies_arrow.PNG")
        if self.position.x > pygame.display.get_surface().get_width() / 2:
            self.image = pygame.transform.flip(self.image, True, False)
        self.rect.center = (self.position.x, self.position.y)
        self.velocity = velocity.copy()
        self.alive = True
        
    def update(self, dt, world):
        if self.alive:
            Triggerable.update(self, dt, world)
            if self.triggered:
                self.velocity += GRAVITY * dt
                self.position += self.velocity * dt
        
            self.rect.center = (self.position.x, self.position.y)
        
            if self.collision_check(world):
                world.player.change_state(world, world.player.states["dead"])
        self.check_onscreen(world)
    
class BananaPickup(pygame.sprite.DirtySprite, Collidable):
    def __init__(self, startpos, velocity=Vector(0.0,0.0)):
        pygame.sprite.DirtySprite.__init__(self)
        self.dirty = 1
        self.position = startpos
        self.image, self.rect = data.load_image("banana.PNG")
        self.rect.center = (self.position.x, self.position.y)
        self.velocity = velocity.copy()
        self.alive = True
        
    def update(self, dt, world):
        self.rect.center = (self.position.x, self.position.y)

        if self.alive and self.collision_check(world):
            sound.play_sound("powerup.wav");  
            self.image = pygame.Surface((0,0))
            self.rect = self.image.get_rect()
            self.image.set_alpha(0)
            self.dirty = 1
            self.alive = False   
            world.player.add_life(world)
               
class Fly(pygame.sprite.DirtySprite, Collidable):
    def __init__(self, startpos, velocity=Vector(0.1,0.0), dx=100):
        pygame.sprite.DirtySprite.__init__(self)
        self.image, self.rect = data.load_image("fly.PNG")
        self.dirty = 2
        self.position = startpos.copy()
        self.velocity = velocity.copy()
        self.rect.center = (self.position.x, self.position.y)
        self.dx = dx
        self.distance_travelled = 0
                    
    def update(self, dt, world):
        self.position += self.velocity * dt
        self.distance_travelled += (self.velocity * dt).magnitude()
        
        if self.distance_travelled > self.dx:
            self.distance_travelled = 0
            self.velocity = -self.velocity
        
        if self.collision_check(world):
                world.player.change_state(world, world.player.states["dead"])
        
        self.rect.center = (self.position.x, self.position.y)
        
class Bird(pygame.sprite.DirtySprite, Collidable):
    def __init__(self, shit_interval, startpos, velocity=Vector(0.1, 0.0), dx=200):
        pygame.sprite.DirtySprite.__init__(self)
        self.image, self.rect = data.load_image("enemies_bird.PNG", [0, 255, 255])
        self.dirty = 2
        self.position = startpos.copy()
        self.velocity = velocity.copy()
        self.rect.center = (self.position.x, self.position.y)
        self.dx = dx
        self.distance_travelled = 0
        self.shit_interval = shit_interval
        self.time_since_last_shit = 0
        
        self.shits = []
        
        self.turned = False
        
    def update(self, dt, world):
        self.position += self.velocity * dt
        self.distance_travelled += (self.velocity * dt).magnitude()
        
        if self.distance_travelled > self.dx:
            self.distance_travelled = 0
            self.velocity = -self.velocity
            self.turned = not self.turned
            self.image = pygame.transform.flip(self.image, True, False)
        
        if self.collision_check(world):
                world.player.change_state(world, world.player.states["dead"])
                
        self.time_since_last_shit += dt
        if math.fabs(self.position.x - world.player.position.x) < 5 and self.time_since_last_shit > self.shit_interval:
            self.shit()
        
        self.rect.center = (self.position.x, self.position.y)
    
    def shit(self):
        shit_start_pos = self.position.copy()
        shit_start_pos.y += 20
        if self.turned:
            shit_start_pos.x += 20
        else:
            shit_start_pos.x -= 20
        new_shit = BirdShit(shit_start_pos)
        for spritegroup in self.groups():
            spritegroup.add(new_shit)
        self.time_since_last_shit = 0
        
class BirdShit(pygame.sprite.DirtySprite, Collidable):
    def __init__(self, startpos):
        pygame.sprite.DirtySprite.__init__(self)
        self.position = startpos.copy()
        self.dirty = 2
        self.image, self.rect = data.load_image("enemies_birdshit.PNG", [0, 255, 255])
        self.rect.center = (self.position.x, self.position.y)
        self.velocity = Vector(0.0, 0.0)
        self.alive = True
    
    def update(self, dt, world):
        if self.alive:
            self.velocity += GRAVITY * dt * 10
            self.position += self.velocity * dt
            
            self.rect.center = (self.position.x, self.position.y)
        
            if self.collision_check(world):
                world.player.change_state(world, world.player.states["dead"])
                
        self.check_onscreen(world)