import operator
import math

class Vector:
    def __init__(self, x, y):
        self.x = x
        self.y = y
        
    def __copy__(self):
        return Vector(self.x, self.y)
    
    copy = __copy__
    
    def __repr__(self):
        return 'Vector(%(x)f, %(y)f)' % {'x': self.x, 'y': self.y}
        
    def __eq__(self, other):
        assert isinstance(other, Vector)
        return self.x == other.x and self.y == other.y
    
    def __ne__(self, other):
        return not self.__eq__(other)
    
    def __nonzero__(self):
        return self.x != 0 or self.y != 0
    
    def __add__(self, other):
        assert isinstance(other, Vector)
        return Vector(self.x + other.x, self.y + other.y)
    
    __radd__ = __add__
    
    def __iadd__(self, other):
        assert isinstance(other, Vector)
        self.x += other.x
        self.y += other.y
        return self
    
    def __sub__(self, other):
        assert isinstance(other, Vector)
        return Vector(self.x - other.x, self.y - other.y)
    
    def __rsub__(self, other):
        assert isinstance(other, Vector)
        return Vector(other.x - self.x, other.y - self.y)
    
    def __mul__(self, number):
        return Vector(self.x * number, self.y * number)
    
    __rmul__ = __mul__
    
    def __imul__(self, number):
        self.x *= number
        self.y *= number
        return self
    
    def __div__(self, number):
        return Vector(operator.truediv(self.x, number), operator.truediv(self.y, number))
    
    def __rdiv__(self, number):
        return Vector(operator.truediv(number, self.x), operator.truediv(number, self.y))
    
    def __neg__(self):
        return Vector(-self.x, -self.y)
    
    __pos__ = __copy__
    
    def __abs__(self):
        return math.sqrt(self.x ** 2 + self.y ** 2)
    
    magnitude = __abs__
    
    def magnitude_sq(self):
        return self.x ** 2 + self.y ** 2
    
    def normalize(self):
        l = self.magnitude()
        if l != 0:
            return Vector(self.x / l, self.y / l)
        else:
            return self
    
    def dot(self, other):
        assert isinstance(other, Vector)
        return self.x * other.x + self.y * other.y
    
    def perp(self):
        return Vector(-self.y, self.x).normalize()
    
    def trunc(self, length):
        if self.magnitude_sq() > (length ** 2):
            return self.normalize() * length
        else:
            return self