import pygame
import data
import pygame.transform
import pygame.sprite
import random
from pygame.locals import *
from rope import Rope
from player import Player
from vines import Vine
from vector import Vector
from enemies import *
from levels import Levels
from infobox import *
import sound

class Background(pygame.sprite.DirtySprite):
    def __init__(self, color):
        pygame.sprite.DirtySprite.__init__(self)
        self.image = pygame.Surface((pygame.display.get_surface().get_width(), pygame.display.get_surface().get_height()))
        self.image.convert()
        self.image.fill(color)
        self.rect = self.image.get_rect()
        
class Cloud(pygame.sprite.DirtySprite):
    def __init__(self, startpos=Vector(-1.0,0.0), flip=False):
        pygame.sprite.DirtySprite.__init__(self)
        self.image, self.rect = data.load_image("cloud.PNG")
        if flip == True:
            self.image = pygame.transform.flip(self.image, True, False)
        self.position = startpos
        self.velocity = Vector(float(random.randrange(1, 5, 1))/100,0.0)
        self.dirty = 2
        
    def update(self, dt, world):
        self.position += self.velocity
        if self.position.x < 0 or self.position.x > pygame.display.get_surface().get_width():
            self.position.x = 0
            self.velocity = Vector(float(random.randrange(1, 5, 1))/100,0.0)
            self.position.y = random.randint(30, pygame.display.get_surface().get_height()-30)
        self.rect.center = (self.position.x, self.position.y)

class BananaDisplay(pygame.sprite.DirtySprite):
    def __init__(self, number):
        pygame.sprite.DirtySprite.__init__(self)
        self.number = number
        self.uneatenimage, self.uneatenimage_rect = data.load_image("banana.PNG") 
        self.eatenimage, self.eatenimage_rect = data.load_image("banana_eaten.PNG")
        self.image, self.rect = self.uneatenimage, self.uneatenimage_rect
        self.rect.move_ip((number-1)*50, 0)
        
    def update(self, dt, world):
        if world.player.lives >= self.number and self.image == self.eatenimage:
            self.image = self.uneatenimage
            self.dirty = 1
        elif world.player.lives < self.number and self.image == self.uneatenimage:
            self.image = self.eatenimage
            self.dirty = 1
            
class Tree(pygame.sprite.DirtySprite):
    def __init__(self, flip):
        pygame.sprite.DirtySprite.__init__(self)
        self.image, self.rect = data.load_image("tree.PNG", [0, 255, 255])
        if flip:
            self.image = pygame.transform.flip(self.image, True, True)
            self.rect.right = pygame.display.get_surface().get_width()
    
class Branch(pygame.sprite.DirtySprite):
    def __init__(self, attachposition):
        pygame.sprite.DirtySprite.__init__(self)
        
        self.image, self.rect = data.load_image("branch.PNG")
        
        if attachposition.x > 400:
            self.image = pygame.transform.flip(self.image, True, False)
            self.rect.topleft = (attachposition.x, attachposition.y)
            self.rect.move_ip(-50, -10)
        else:
            self.rect.topright = (attachposition.x, attachposition.y)
            self.rect.move_ip(50, -10)
        pass
    
class ToldYaException(Exception):
    def __init__(self):
        pass

class World:  
    def __init__(self):        
        self.level = 0

        self.levels = Levels().get_levels()
        self.climb_height = 0
        
        self.background = Background([150, 200, 255])
        self.clouds = [Cloud(Vector(300,400),False), Cloud(Vector(0,200),True), Cloud(Vector(0,600),True), Cloud()]        
        self.rope = Rope(400)
        
        self.exitmenu_open = False
        self.exitmenu = HugeInfoBox((400, 300), "Want to exit? (Y/N)")
        self.deathmenu_open = False
        self.deathmenu = [HugeInfoBox((400,220), "(N)ew game"),
                          HugeInfoBox((400,300), "(C)ontinue"),
                          HugeInfoBox((400,380), "(E)xit")]
        
        
        self.vines = [Vine(Vector(200, 200), 200), Vine(Vector(600, 100), 250), Vine(Vector(250, 450), 150)]
        self.enemies, self.vines, self.special_sprites = self.levels[self.level].initialize()
        
        self.previous_keys = pygame.key.get_pressed()
        
        self.player = Player(Vector(self.rope.x, pygame.display.get_surface().get_height())) 
        
        self.bananas = []
        self.max_lives = 5
        for life_index in range(1,self.max_lives+1):
            self.bananas.append(BananaDisplay(life_index))  
            
        self.stat_box = StatBox()
            
        self.spritegroup = pygame.sprite.LayeredDirty()
        self.init_sprite_group()
        
        self.width, self.height = pygame.display.get_surface().get_width(), pygame.display.get_surface().get_height()
        
        sound.play_music("music.ogg");
        
    def reset(self, player_y):
        self.player.set_y(player_y)
        self.enemies, self.vines, self.special_sprites = self.levels[self.level].initialize()
        self.init_sprite_group()
        
    def level_up(self):
        self.level += 1
        self.reset(self.height)

    def level_down(self):
        self.level -= 1
        self.reset(0)
     
    def init_sprite_group(self):
        self.spritegroup.empty()
        
        self.spritegroup.add(self.background)
        
        for cloud in self.clouds: self.spritegroup.add(cloud)
        
        self.spritegroup.add(Tree(False))
        self.spritegroup.add(Tree(True))
        
        self.spritegroup.add(self.rope)
        
        for special in self.special_sprites: self.spritegroup.add(special)
        
        self.spritegroup.add(self.stat_box)
               
        for vine in self.vines: self.spritegroup.add(vine)
        
        for vine in self.vines: self.spritegroup.add(Branch(vine.attachpoint.position))
        
        for banana in self.bananas: self.spritegroup.add(banana)
        
        for enemy in self.enemies: self.spritegroup.add(enemy)
        
        self.spritegroup.add(self.player)
        
        
    def check_level(self):
        if self.player.position.y <= 0:
            try:
                self.level_up()
            except:
                raise ToldYaException
        if(self.player.position.y + self.player.velocity.y > self.height):
            self.player.position.y = self.height
            self.player.velocity.y = 0
        
    def update(self, dt):
        if(not self.exitmenu_open and not self.deathmenu_open):
            self.spritegroup.update(dt, self)
            self.check_level()   
            self.climb_height = ((self.level + 1) * self.height - self.player.position.y) / 60  
          
    def draw(self, dest):
        return self.spritegroup.draw(dest)
    
    def open_Exitmenu(self):
        self.spritegroup.add(self.exitmenu)
        self.exitmenu_open = True
        
    def close_Exitmenu(self):
        self.spritegroup.remove(self.exitmenu)
        self.exitmenu_open = False
    
    def open_Deathmenu(self):
        for menu in self.deathmenu: self.spritegroup.add(menu)
        self.deathmenu_open = True
        
    def close_Deathmenu(self):
        for menu in self.deathmenu: self.spritegroup.remove(menu)
        self.deathmenu_open = False
        
    def handle_input(self, keystates):
        if keystates[K_ESCAPE] and not self.deathmenu_open:
            if not self.exitmenu_open:
                self.open_Exitmenu()
        
        if(self.exitmenu_open or self.deathmenu_open):
            if(self.handle_menuinput(keystates)):
                return True
        
        else:
            self.player.handle_input(self, keystates, self.previous_keys)
            self.previous_keys = keystates
            
    def handle_menuinput(self, keystates):
        if (keystates[K_y] and self.exitmenu_open) or (keystates[K_e] and self.deathmenu_open):
            return True
        
        if keystates[K_n] and self.exitmenu_open:
            self.close_Exitmenu()
            
        if keystates[K_c] and self.deathmenu_open:
            self.player.lives = 3
            self.close_Deathmenu()
        
        if keystates[K_n] and self.deathmenu_open:
            self.level = 0
            self.player.lives = 3
            self.reset(self.height)
            self.close_Deathmenu()
        
    def get_closest_vinelink(self, position):
        closest = None
        closest_distance_sq = -1
        closest_vine = None
        vine_changed = False
        for vine in self.vines:
            for link in vine.links:
                distance_sq = (link.position - position).magnitude_sq()
                if distance_sq < closest_distance_sq or closest_distance_sq == -1:
                    closest = link
                    closest_distance_sq = distance_sq
                    vine_changed = True
                    continue
            if vine_changed:
                vine_changed = False
                closest_vine = vine
        return closest, closest_vine
            